! =================================================================================================================================
! MODULE 	: constantes_soil
!
! CONTACT       : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE       : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF         "constantes_soil" module contains subroutine to initialize the parameters related to soil and hydrology.
!!
!!\n DESCRIPTION : "constantes_soil" module contains subroutine to initialize the parameters related to soil and hydrology.
!!                 This module alos USE constates_soil and can therfor be used to acces the subroutines and the constantes.
!!                 The constantes declarations can also be used seperatly with "USE constantes_soil_var".
!!
!! RECENT CHANGE(S): 
!!
!! REFERENCE(S)	:
!!
!! SVN          :
!! $HeadURL: $
!! $Date: $
!! $Revision: $
!! \n
!_ ================================================================================================================================

MODULE constantes_soil

  USE constantes_soil_var
  USE constantes
  USE ioipsl_para 

  IMPLICIT NONE

CONTAINS

  
!! ================================================================================================================================
!! SUBROUTINE   : config_soil_parameters
!!
!>\BRIEF        This subroutine reads in the configuration file all the parameters related to soil and hydrology. 
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): 
!!
!! REFERENCE(S) :
!!
!! FLOWCHART    :
!! \n
!_ ================================================================================================================================

   SUBROUTINE config_soil_parameters(active_flags)
     
     USE ioipsl

     IMPLICIT NONE

     !! 0. Variables and parameters declaration

     !! 0.1 Input variables

     TYPE(control_type), INTENT(in) :: active_flags            !! What parts of the code are activated ? (true/false)  

    !! 0.4 Local variables 

     INTEGER(i_std), PARAMETER      :: error_level = 3         !! Switch to 2 to turn fatal errors into warnings.(1-3, unitless)
     LOGICAL, SAVE                  :: first_call = .TRUE.     !! To keep first call trace (true/false)
!$OMP THREADPRIVATE(first_call)

!_ ================================================================================================================================
     
     IF ( first_call ) THEN
        
        !Config Key   = DRY_SOIL_HEAT_CAPACITY
        !Config Desc  = Dry soil Heat capacity of soils
        !Config If    = OK_SECHIBA 
        !Config Def   = 1.80e+6
        !Config Help  = Values taken from : PIELKE,'MESOSCALE METEOROLOGICAL MODELING',P.384.
        !Config Units = [J.m^{-3}.K^{-1}] 
        CALL getin_p("DRY_SOIL_HEAT_CAPACITY",so_capa_dry)
        
        !! Check parameter value (correct range)
        IF ( so_capa_dry <= zero ) THEN
           CALL ipslerr_p(error_level, "config_soil_parameters.", &
                &     "Wrong parameter value for DRY_SOIL_HEAT_CAPACITY.", &
                &     "This parameter should be positive. ", &
                &     "Please, check parameter value in run.def. ")
        END IF
        

        !Config Key   = DRY_SOIL_HEAT_COND
        !Config Desc  = Dry soil Thermal Conductivity of soils
        !Config If    = OK_SECHIBA
        !Config Def   = 0.40 
        !Config Help  = Values taken from : PIELKE,'MESOSCALE METEOROLOGICAL MODELING',P.384.
        !Config Units = [W.m^{-2}.K^{-1}] 
        CALL getin_p("DRY_SOIL_HEAT_COND",so_cond_dry)

        !! Check parameter value (correct range)
        IF ( so_cond_dry <= zero ) THEN
           CALL ipslerr_p(error_level, "config_soil_parameters.", &
                &     "Wrong parameter value for DRY_SOIL_HEAT_COND.", &
                &     "This parameter should be positive. ", &
                &     "Please, check parameter value in run.def. ")
        END IF


        !Config Key   = WET_SOIL_HEAT_CAPACITY
        !Config Desc  = Wet soil Heat capacity of soils 
        !Config If    = OK_SECHIBA
        !Config Def   = 3.03e+6
        !Config Help  = 
        !Config Units = [J.m^{-3}.K^{-1}]
        CALL getin_p("WET_SOIL_HEAT_CAPACITY",so_capa_wet)

        !! Check parameter value (correct range)
        IF ( so_capa_wet <= zero ) THEN
           CALL ipslerr_p(error_level, "config_soil_parameters.", &
               &     "Wrong parameter value for WET_SOIL_HEAT_CAPACITY.", &
               &     "This parameter should be positive. ", &
               &     "Please, check parameter value in run.def. ")
        END IF


        !Config Key   = WET_SOIL_HEAT_COND
        !Config Desc  = Wet soil Thermal Conductivity of soils
        !Config If    = OK_SECHIBA 
        !Config Def   = 1.89 
        !Config Help  = 
        !Config Units = [W.m^{-2}.K^{-1}]
        CALL getin_p("WET_SOIL_HEAT_COND",so_cond_wet)

        !! Check parameter value (correct range)
        IF ( so_cond_wet <= zero ) THEN
           CALL ipslerr_p(error_level, "config_soil_parameters.", &
               &     "Wrong parameter value for WET_SOIL_HEAT_COND.", &
               &     "This parameter should be positive. ", &
               &     "Please, check parameter value in run.def. ")
        END IF


        !Config Key   = SNOW_HEAT_COND
        !Config Desc  = Thermal Conductivity of snow
        !Config If    = OK_SECHIBA  
        !Config Def   = 0.3
        !Config Help  = 
        !Config Units = [W.m^{-2}.K^{-1}]
        CALL getin_p("SNOW_HEAT_COND",sn_cond)

        !! Check
        IF ( sn_cond <= zero ) THEN
           CALL ipslerr_p(error_level, "config_soil_parameters.", &
               &     "Wrong parameter value for SNOW_HEAT_COND.", &
               &     "This parameter should be positive. ", &
               &     "Please, check parameter value in run.def. ")
        END IF


        !Config Key   = SNOW_DENSITY
        !Config Desc  = Snow density for the soil thermodynamics 
        !Config If    = OK_SECHIBA 
        !Config Def   = 330.0
        !Config Help  = 
        !Config Units = [-] 
        CALL getin_p("SNOW_DENSITY",sn_dens)
        
        !! Check parameter value (correct range)
        IF ( sn_dens <= zero ) THEN
          CALL ipslerr_p(error_level, "config_soil_parameters.", &
               &     "Wrong parameter value for SNOW_DENSITY.", &
               &     "This parameter should be positive. ", &
               &     "Please, check parameter value in run.def. ")
        END IF


        !! Calculation of snow capacity
        !! If sn_dens is redefined by the user, sn_capa needs to be reset
        sn_capa = 2100.0_r_std*sn_dens


        !Config Key   = NOBIO_WATER_CAPAC_VOLUMETRI
        !Config Desc  = 
        !Config If    = OK_SECHIBA and .NOT.(HYDROL_CWRR)
        !Config Def   = 150.
        !Config Help  = 
        !Config Units = [s/m^2]
        CALL getin_p('NOBIO_WATER_CAPAC_VOLUMETRI',mx_eau_nobio)

       !! Check parameter value (correct range)
        IF ( mx_eau_nobio <= zero ) THEN
           CALL ipslerr_p(error_level, "config_soil_parameters.", &
               &     "Wrong parameter value for NOBIO_WATER_CAPAC_VOLUMETRI.", &
               &     "This parameter should be positive. ", &
               &     "Please, check parameter value in run.def. ")
        END IF


        !Config Key   = SECHIBA_QSINT 
        !Config Desc  = Interception reservoir coefficient
        !Config If    = OK_SECHIBA 
        !Config Def   = 0.1
        !Config Help  = Transforms leaf area index into size of interception reservoir
        !Config         for slowproc_derivvar or stomate
        !Config Units = [m]
        CALL getin_p('SECHIBA_QSINT',qsintcst)

        !! Check parameter value (correct range)
        IF ( qsintcst <= zero ) THEN
           CALL ipslerr_p(error_level, "config_soil_parameters.", &
                &     "Wrong parameter value for SECHIBA_QSINT.", &
                &     "This parameter should be positive. ", &
                &     "Please, check parameter value in run.def. ")
        END IF


        IF ( .NOT.(active_flags%hydrol_cwrr) ) THEN
           
           !Config Key   = CHOISNEL_DIFF_MIN
           !Config Desc  = Diffusion constant for the slow regime
           !Config If    = OK_SECHIBA and .NOT.(HYDROL_CWRR)
           !Config Def   = 0.001
           !Config Help  = 
           !Config Units = [kg/m^2/dt]
           CALL getin_p('CHOISNEL_DIFF_MIN',min_drain)

           !! Check parameter value (correct range)
           IF ( min_drain <= zero ) THEN
              CALL ipslerr_p(error_level, "config_soil_parameters.", &
                   &     "Wrong parameter value for CHOISNEL_DIFF_MIN.", &
                   &     "This parameter should be positive. ", &
                   &     "Please, check parameter value in run.def. ")
            END IF


           !Config Key   = CHOISNEL_DIFF_MAX
           !Config Desc  = Diffusion constant for the fast regime
           !Config If    = OK_SECHIBA and .NOT.(HYDROL_CWRR)
           !Config Def   = 0.1
           !Config Help  = 
           !Config Units = [kg/m^2/dt]
           CALL getin_p('CHOISNEL_DIFF_MAX',max_drain)

           !! Check parameter value (correct range)
           IF (  ( max_drain <= zero ) .OR. ( max_drain <= min_drain ) ) THEN
              CALL ipslerr_p(error_level, "config_soil_parameters.", &
                   &     "Wrong parameter value for CHOISNEL_DIFF_MAX.", &
                   &     "This parameter should be positive or greater than CHOISNEL_DIFF_MIN.", &
                   &     "Please, check parameter value in run.def. ")
           END IF


           !Config Key   = CHOISNEL_DIFF_EXP
           !Config Desc  = The exponential in the diffusion law
           !Config If    = OK_SECHIBA and .NOT.(HYDROL_CWRR)
           !Config Def   = 1.5
           !Config Help  = 
           !Config Units = [-]
           CALL getin_p('CHOISNEL_DIFF_EXP',exp_drain)
           
           !! Check parameter value (correct range)
           IF ( exp_drain <= zero ) THEN
              CALL ipslerr_p(error_level, "config_soil_parameters.", &
                   &     "Wrong parameter value for CHOISNEL_DIFF_EXP.", &
                   &     "This parameter should be positive. ", &
                   &     "Please, check parameter value in run.def. ")
           END IF


           !Config Key   = CHOISNEL_RSOL_CSTE
           !Config Desc  = Constant in the computation of resistance for bare  soil evaporation 
           !Config If    = OK_SECHIBA and .NOT.(HYDROL_CWRR)
           !Config Def   = 33.E3
           !Config Help  = 
           !Config Units = [s/m^2]
           CALL getin_p('CHOISNEL_RSOL_CSTE',rsol_cste)

           !! Check parameter value (correct range)
           IF ( rsol_cste <= zero ) THEN
              CALL ipslerr_p(error_level, "config_soil_parameters.", &
                   &     "Wrong parameter value for CHOISNEL_RSOL_CSTE.", &
                   &     "This parameter should be positive. ", &
                   &     "Please, check parameter value in run.def. ")
           END IF


           !Config Key   = HCRIT_LITTER
           !Config Desc  = Scaling depth for litter humidity
           !Config If    = OK_SECHIBA and .NOT.(HYDROL_CWRR) 
           !Config Def   = 0.08 
           !Config Help  = 
           !Config Units = [m]
           CALL getin_p('HCRIT_LITTER',hcrit_litter)

           !! Check parameter value (correct range)
           IF ( hcrit_litter <= zero ) THEN
              CALL ipslerr_p(error_level, "config_soil_parameters.", &
                   &     "Wrong parameter value for HCRIT_LITTER.", &
                   &     "This parameter should be positive. ", &
                   &     "Please, check parameter value in run.def. ")
           END IF


        END IF

        first_call =.FALSE.
        
     ENDIF
     
   END SUBROUTINE config_soil_parameters
   

END MODULE constantes_soil
